@@ -38,8 +38,10 @@ import de.intarsys.tools.stream.StreamTools;
 import de.intarsys.tools.string.StringTools;
 
 /**
- * Apple TT specification
- * http://developer.apple.com/fonts/TTRefMan/RM06/Chap6.html
+ * Apple TT specification<br/>
+ * http://developer.apple.com/fonts/TTRefMan/RM06/Chap6.html <br/>
+ * Microsoft OpenType spec <br/>
+ * http://www.microsoft.com/typography/otspec/otff.htm
  */
 public class TTFontParser {
 	// todo support indexed fonts
@@ -687,11 +689,7 @@ public class TTFontParser {
 
 	public void readTables(TTFont font, IRandomAccess random)
 			throws IOException {
-		// this is not true for all valid font files, skip version check
-		// if (readInt(is) != 0x00010000) {
-		// throw new IOException("not a valid TTF file.");
-		// }
-		readInt(random);
+		font.setSfntVersion(readInt(random));
 
 		int tableCount = readUShort(random);
 
