@@ -58,7 +58,6 @@ import org.eclipse.swt.graphics.Transform;
 import de.intarsys.cwt.common.BlendMode;
 import de.intarsys.cwt.common.IGraphicsObject;
 import de.intarsys.cwt.common.IPaint;
-import de.intarsys.cwt.common.ShapeWrapper;
 import de.intarsys.cwt.environment.IGraphicsContext;
 import de.intarsys.cwt.environment.IGraphicsEnvironment;
 import de.intarsys.cwt.image.IImage;
@@ -165,6 +164,10 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 		return st;
 	}
 
+	private boolean applyAlphaBackground;
+
+	private boolean applyAlphaForeground;
+
 	private boolean applyTransform = false;
 
 	private Color backgroundColor;
@@ -183,8 +186,6 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 
 	private Device device;
 
-	private Shape deviceClip;
-
 	private float[] elements = new float[6];
 
 	private double[] flatmatrix = new double[6];
@@ -199,10 +200,6 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 
 	private GC gc;
 
-	private boolean setAlphaBackground;
-
-	private boolean setAlphaForeground;
-
 	private int swtBackgroundAlpha;
 
 	private org.eclipse.swt.graphics.Color swtBackgroundColor;
@@ -223,8 +220,8 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 		swtForegroundColor = null;
 		foregroundColor = null;
 		gc = paramGC;
-		setAlphaForeground = false;
-		setAlphaBackground = false;
+		applyAlphaForeground = false;
+		applyAlphaBackground = false;
 
 		swtTransform = new Transform(paramDevice);
 		paramGC.getTransform(swtTransform);
@@ -233,7 +230,8 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 
 		// this is the "default" clipping
 		Rectangle2D rect = toRectangle2D(paramGC.getClipping());
-		currentClip = ApplySpaceChangeShape.create(rect, currentTransform);
+		currentClip = ApplySpaceChangeShape.create(new Area(rect),
+				currentTransform);
 
 		// when setting to SWT.DEFAULT here curved shapes (like letters) will
 		// look awful on gtk
@@ -241,6 +239,48 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 		paramGC.setInterpolation(SWT.NONE);
 	}
 
+	protected void applyAlphaBackground() {
+		if (applyAlphaBackground) {
+			applyAlphaBackground = false;
+			if (gc.getAlpha() != swtBackgroundAlpha) {
+				gc.setAlpha(swtBackgroundAlpha);
+				if (swtForegroundAlpha != swtBackgroundAlpha) {
+					applyAlphaForeground = true;
+				}
+			}
+		}
+	}
+
+	protected void applyAlphaForeground() {
+		if (applyAlphaForeground) {
+			applyAlphaForeground = false;
+			if (gc.getAlpha() != swtForegroundAlpha) {
+				gc.setAlpha(swtForegroundAlpha);
+				if (swtBackgroundAlpha != swtForegroundAlpha) {
+					applyAlphaBackground = true;
+				}
+			}
+		}
+	}
+
+	protected void applyClip() {
+		if (currentClip == null) {
+			gc.setClipping((Region) null);
+		} else {
+			// transform gc before clip is set
+			applyTransform();
+			PathIterator pathIterator = currentClip.getPathIterator(null);
+			if (pathIterator.getWindingRule() == PathIterator.WIND_EVEN_ODD) {
+				gc.setFillRule(SWT.FILL_EVEN_ODD);
+			} else {
+				gc.setFillRule(SWT.FILL_WINDING);
+			}
+			Path clipPath = toPath(device, pathIterator);
+			gc.setClipping(clipPath);
+			clipPath.dispose();
+		}
+	}
+
 	protected void applyTransform() {
 		if (applyTransform) {
 			applyTransform = false;
@@ -254,20 +294,25 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	}
 
 	public void clip(Shape shape) {
-		if (currentClip != null) {
-			Area newShape = new Area(shape);
-			Area intersection = new Area(currentClip);
-			intersection.intersect(newShape);
-			shape = new ShapeWrapper(intersection);
+		Area tempShape = ShapeTools.createArea(shape, false);
+		if (currentClip == null) {
+			currentClip = ApplySpaceChangeShape.create(tempShape,
+					currentTransform);
+		} else {
+			Area tempClip = ShapeTools.createArea(currentClip, true);
+			tempClip.intersect(tempShape);
+			currentClip = ApplySpaceChangeShape.create(tempClip,
+					currentTransform);
 		}
-		setClip(shape);
+		applyClip();
 	}
 
 	protected Path createPath(Shape s) {
 		Path path;
 		if (s instanceof IGraphicsObject) {
 			path = (Path) ((IGraphicsObject) s).getData();
-			if (path != null) {
+			if (path != null && !path.isDisposed()
+					&& !path.getDevice().isDisposed()) {
 				return path;
 			}
 		}
@@ -318,15 +363,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	}
 
 	public void draw(Shape s) {
-		if (setAlphaForeground) {
-			setAlphaForeground = false;
-			if (gc.getAlpha() != swtForegroundAlpha) {
-				gc.setAlpha(swtForegroundAlpha);
-				if (swtBackgroundAlpha != swtForegroundAlpha) {
-					setAlphaBackground = true;
-				}
-			}
-		}
+		applyAlphaForeground();
 		if (s instanceof TransformedShape && gc.getForegroundPattern() == null) {
 			AffineTransform awtShapeTransform = ((TransformedShape) s)
 					.getTransform();
@@ -361,6 +398,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	}
 
 	public void drawImage(IImage image, float x, float y) {
+		applyAlphaBackground();
 		applyTransform();
 		image.drawFromGraphicsContext(this, x, y);
 	}
@@ -368,15 +406,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	public void drawString(String s, float x, float y) {
 		applyTransform();
 		boolean reset;
-		if (setAlphaForeground) {
-			setAlphaForeground = false;
-			if (gc.getAlpha() != swtForegroundAlpha) {
-				gc.setAlpha(swtForegroundAlpha);
-				if (swtBackgroundAlpha != swtForegroundAlpha) {
-					setAlphaBackground = true;
-				}
-			}
-		}
+		applyAlphaForeground();
 		reset = false;
 		if (fontTransform != null) {
 			float[] point;
@@ -406,15 +436,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	}
 
 	public void fill(Shape s) {
-		if (setAlphaBackground) {
-			setAlphaBackground = false;
-			if (gc.getAlpha() != swtBackgroundAlpha) {
-				gc.setAlpha(swtBackgroundAlpha);
-				if (swtForegroundAlpha != swtBackgroundAlpha) {
-					setAlphaForeground = true;
-				}
-			}
-		}
+		applyAlphaBackground();
 		if (s instanceof TransformedShape && gc.getBackgroundPattern() == null) {
 			AffineTransform awtShapeTransform = ((TransformedShape) s)
 					.getTransform();
@@ -474,6 +496,9 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	}
 
 	public Shape getClip() {
+		if (currentClip == null) {
+			return null;
+		}
 		return (Shape) currentClip.clone();
 	}
 
@@ -588,7 +613,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	public void setBackgroundColor(Color color) {
 		backgroundColor = color;
 		swtBackgroundAlpha = color.getAlpha();
-		setAlphaBackground = true;
+		applyAlphaBackground = true;
 		org.eclipse.swt.graphics.Color newColor = new org.eclipse.swt.graphics.Color(
 				device, color.getRed(), color.getGreen(), color.getBlue());
 		setBackgroundColor(newColor);
@@ -605,7 +630,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 
 	public void setBackgroundPaint(IPaint paint) {
 		swtBackgroundAlpha = (int) (paint.getAlphaValue() * 255);
-		setAlphaBackground = true;
+		applyAlphaBackground = true;
 		paint.setBackgroundPaintFromGraphicsContext(this);
 	}
 
@@ -624,20 +649,11 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	public void setClip(Shape shape) {
 		if (shape == null) {
 			currentClip = null;
-			gc.setClipping((Region) null);
-			return;
 		} else {
-			currentClip = ApplySpaceChangeShape.create(shape, currentTransform);
+			currentClip = ApplySpaceChangeShape.create(ShapeTools
+					.createArea(shape, false), currentTransform);
 		}
-		// transform gc before clip is set
-		applyTransform();
-		Path clipPath = createPath(shape);
-		if (ShapeTools.getWindingRule(shape) == PathIterator.WIND_EVEN_ODD) {
-			gc.setFillRule(SWT.FILL_EVEN_ODD);
-		} else {
-			gc.setFillRule(SWT.FILL_WINDING);
-		}
-		gc.setClipping(clipPath);
+		applyClip();
 	}
 
 	public void setFont(Font awtFont) {
@@ -670,7 +686,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 	public void setForegroundColor(Color color) {
 		foregroundColor = color;
 		swtForegroundAlpha = color.getAlpha();
-		setAlphaForeground = true;
+		applyAlphaForeground = true;
 		org.eclipse.swt.graphics.Color newColor = new org.eclipse.swt.graphics.Color(
 				device, color.getRed(), color.getGreen(), color.getBlue());
 		setForegroundColor(newColor);
@@ -687,7 +703,7 @@ public class CwtSwtGraphicsContext implements IGraphicsContext {
 
 	public void setForegroundPaint(IPaint paint) {
 		swtForegroundAlpha = (int) (paint.getAlphaValue() * 255);
-		setAlphaForeground = true;
+		applyAlphaForeground = true;
 		paint.setForegroundPaintFromGraphicsContext(this);
 	}
 
